/* ****************************************************************************
  This file is part of KBabel

  Copyright (C) 2002-2003 by Marco Wegner <mail@marcowegner.de>
  Copyright (C) 2005, 2006 by Nicolas GOUTTE <goutte@kde.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  In addition, as a special exception, the copyright holders give
  permission to link the code of this program with any edition of
  the TQt library by Trolltech AS, Norway (or with modified versions
  of TQt that use the same license as TQt), and distribute linked
  combinations including the two.  You must obey the GNU General
  Public License in all respects for all of the code used other than
  TQt. If you modify this file, you may extend this exception to
  your version of the file, but you are not obligated to do so.  If
  you do not wish to do so, delete this exception statement from
  your version.

**************************************************************************** */


// TQt include files
#include <tqcheckbox.h>
#include <tqcombobox.h>
#include <tqfileinfo.h>
#include <tqframe.h>
#include <tqlabel.h>
#include <tqlayout.h>
#include <tqlistbox.h>
#include <tqpushbutton.h>
#include <tqstring.h>
#include <tqstringlist.h>
#include <tqtextedit.h>
// KDE include files
#include <tdeconfig.h>
#include <kdebug.h>
#include <tdeglobal.h>
#include <tdelocale.h>
#include <kprocess.h>
#include <tdetempfile.h>
#include <tdemessagebox.h>
#include <kstringhandler.h>
// Project specific include files
#include "svndialog.h"


SVNDialog::SVNDialog( SVN::Command cmd, TQWidget * parent, TDESharedConfig* config )
  : KDialog( parent, "SVN DIALOG", true ), m_tempFile( 0 ), m_config( config )
{
  _cmd = cmd;
  p=0L;
  setCaption( i18n( "SVN Dialog" ) );

  TQString temp;

  TQVBoxLayout * layout = new TQVBoxLayout( this, 6, 6, "MAIN LAYOUT" );

  // Set the label's text depending on the SVN command.
  switch ( cmd ) {
    case SVN::Update:
      temp = i18n( "Update the following files:" );
      break;
    case SVN::Commit:
      temp = i18n( "Commit the following files:" );
      break;
    case SVN::StatusRemote:
      temp = i18n( "Get remote status for the following files:" );
      break;
    case SVN::StatusLocal:
      temp = i18n( "Get local status for the following files:" );
      break;
    case SVN::Diff:
      temp = i18n( "Get diff for the following files:" );
      break;
    case SVN::Info:
      temp = i18n( "Get information for the following files:" );
      break;
  }
  layout->addWidget( new TQLabel( temp, this ) );

  // Widget for showing the list of files.
  filebox = new TQListBox( this );
  layout->addWidget( filebox );

  // Add special widgets for 'svn commit'.
  if ( cmd == SVN::Commit ) {
    TQLabel * label;

    // Combobox for displaying old log messages.
    label = new TQLabel( i18n( "&Old messages:" ), this );
    oldMessages = new TQComboBox( this );
    oldMessages->setDuplicatesEnabled( false );
    label->setBuddy( oldMessages );
    layout->addWidget( label );
    layout->addWidget( oldMessages );

    // Textfield for entering a log message.
    label = new TQLabel( i18n( "&Log message:" ), this );
    logedit = new TQTextEdit( this );
    label->setBuddy( logedit );
    layout->addWidget( label );
    layout->addWidget( logedit );

    connect( oldMessages, TQT_SIGNAL( activated( int ) ),
      this, TQT_SLOT( slotComboActivated( int ) ) );
  }

  TQHBoxLayout * buttons = new TQHBoxLayout( 0, 0, 6, "BUTTON LAYOUT" );
  // Add special buttons for 'svn commit'.
  if ( cmd == SVN::Commit ) {
    autoAddBox = new TQCheckBox( i18n( "Auto&matically add files if necessary" ), this );
    buttons->addWidget( autoAddBox );
  }
  buttons->addItem( new TQSpacerItem( 1, 0, TQSizePolicy::Expanding, TQSizePolicy::Minimum ) );

  // Set the main button's text depending on the SVN comand.
  switch ( cmd ) {
    case SVN::Update:
      temp = i18n( "&Update" );
      break;
    case SVN::Commit:
      temp = i18n( "&Commit" );
      break;
    case SVN::StatusRemote:
    case SVN::StatusLocal:
      temp = i18n( "&Get Status" );
      break;
    case SVN::Diff:
      temp = i18n( "&Get Diff" );
      break;
    case SVN::Info:
      temp = i18n( "&Get Information" );
      break;
  }
  mainBtn = new TQPushButton( temp, this );
  mainBtn->setDefault( true );
  buttons->addWidget( mainBtn );

  cancelBtn = new TQPushButton( i18n( "C&ancel" ), this );
  buttons->addWidget( cancelBtn );
  layout->addLayout( buttons );

  TQFrame * line = new TQFrame( this );
  line->setFrameStyle( TQFrame::HLine | TQFrame::Sunken );
  layout->addWidget( line );

  layout->addWidget( new TQLabel( i18n( "Command output:" ), this ) );

  output = new TQTextEdit( this );
  output->setReadOnly( true );
  layout->addWidget( output );

  resize( TQSize( 600, 450 ).expandedTo( minimumSizeHint( ) ) );

  if ( cmd == SVN::Commit )
    logedit->setFocus( );

  readSettings( );

  connect( mainBtn, TQT_SIGNAL( clicked( ) ), this, TQT_SLOT( slotExecuteCommand( ) ) );
  connect( cancelBtn, TQT_SIGNAL( clicked( ) ), this, TQT_SLOT( reject( ) ) );
}

void SVNDialog::slotComboActivated( int index )
{
  if ( index < 0 || index >= m_logMessages.count() )
    return;
  logedit->setText( m_logMessages[index] );
}

SVNDialog::~SVNDialog()
{
    delete m_tempFile;
    delete p;
}

void SVNDialog::accept( )
{
  saveSettings( );
  KDialog::accept( );
}

void SVNDialog::setFiles( const TQStringList& files )
{
  filebox->insertStringList( files );
}

void SVNDialog::setCommandLine( const TQString& command )
{
  _commandLine = command;
}

void SVNDialog::setAddCommand( const TQString& command )
{
  _addCommand = command;
}

void SVNDialog::slotExecuteCommand( )
{
  // Nothing to do here.
  if ( _commandLine.isEmpty( ) ) return;

  kdDebug() << "Preparing TDEProcess" << endl;

  // Create a new shell process
  p = new TDEProcess;
  p->setUseShell( true, "/bin/sh" );

  if ( _cmd == SVN::Commit ) {
    // Include command for 'svn add'.
    if ( autoAddBox->isChecked( ) && !_addCommand.isEmpty( ) )
      _commandLine.prepend( _addCommand );

    const TQString msg( logedit->text() );

    if ( msg.isEmpty() )
    {
        // A good commit should never have an empty comment, so ask the user if he really wants it.
        const int res = KMessageBox::warningContinueCancel( this,
            i18n( "The commit log message is empty. Do you want to continue?" ) );
        if ( res != KMessageBox::Continue )
            return;
    }

    // Write the commit log message from the input field to a temporary file
    m_tempFile = new KTempFile;
    m_tempFile->setAutoDelete( true );
    TQTextStream* stream = m_tempFile->textStream();
    if ( !stream )
    {
        kdError() << "Could not create TQTextStream for file " << m_tempFile->name();
        delete m_tempFile;
        m_tempFile = 0;
        KMessageBox::error( this, i18n( "Cannot open temporary file for writing. Aborting.") );
        return;
    }
    stream->setEncoding( TQTextStream::UnicodeUTF8 );
    *stream << msg;
    m_tempFile->close();
    
    if ( m_tempFile->status() )
    {
        kdError() << "Could not write to file " << m_tempFile->name();
        delete m_tempFile;
        m_tempFile = 0;
        KMessageBox::error( this, i18n( "Cannot write to temporary file. Aborting.") );
        return;
    }
    
    // Change the command line to have the real name of the temporary file
    _commandLine.replace( "@LOG@FILE@", TDEProcess::quote( m_tempFile->name() ) );

    // Update the list of log messages
    if ( !msg.isEmpty() ) {
      const TQString shortLog = KStringHandler::csqueeze( msg, 80 );
      

      // Remove the message from the list if it already exists
      m_logMessages.remove( msg );
      // Prepend the current message to the list
      m_logMessages.prepend( msg );

      // At this time of the process, we do not need the combobox anymore, so we do not squeeze the changed strings.
    }
  }

  // Set the TDEProcess' command line.
  *p << _commandLine;

  connect( p, TQT_SIGNAL( receivedStdout( TDEProcess*, char*, int ) ),
    this, TQT_SLOT ( slotProcessStdout( TDEProcess*, char*, int ) ) );
  connect( p, TQT_SIGNAL( receivedStderr( TDEProcess*, char*, int ) ),
    this, TQT_SLOT ( slotProcessStderr( TDEProcess*, char*, int ) ) );
  connect( p, TQT_SIGNAL( processExited( TDEProcess* ) ),
    this, TQT_SLOT( slotProcessExited( TDEProcess* ) ) );

  output->append( i18n( "[ Starting command ]" ) );

  if ( p->start( TDEProcess::NotifyOnExit, TDEProcess::Communication( TDEProcess::AllOutput ) ) ) {
    // Disable the main button (and the log edit if in commit mode) to
    // indicate activity.
    mainBtn->setEnabled( false );
    if ( _cmd == SVN::Commit )
      logedit->setEnabled( false );
  } else
  {
    kdError() << "Process could not be started." << endl;
    KMessageBox::error( this, i18n( "The process could not be started." ) );
  }
}

void SVNDialog::slotProcessStdout( TDEProcess*, char * buffer, int len )
{
  output->append( TQString::fromLocal8Bit( buffer, len ) );
  // Set the cursor's position at the end of the output.
  output->setCursorPosition( output->lines( ), 0 );

  // If the command is 'svn status' or 'svn diff' collect the output of stdout.
  if ( ( _cmd == SVN::StatusLocal ) || ( _cmd == SVN::StatusRemote ) || ( _cmd == SVN::Diff ) )
    _statusOutput += TQString::fromLocal8Bit( buffer, len );
}

void SVNDialog::slotProcessStderr( TDEProcess*, char * buffer, int len )
{
  // If an error occurs while executing the command display stderr in
  // another color.
  TQColor oldColor( output->color( ) );
  output->setColor( TQt::red );
  output->append( TQString::fromLocal8Bit( buffer, len ) );
  output->setColor( oldColor );
  output->setCursorPosition( output->lines( ), 0 );
}

void SVNDialog::slotProcessExited( TDEProcess * p )
{
  if ( p->exitStatus( ) )
    output->append( i18n( "[ Exited with status %1 ]" ).arg( p->exitStatus( ) ) );
  else
    output->append( i18n( "[ Finished ]" ) );

  // The command is finished. Now we can reconnect the main button.
  disconnect( mainBtn, 0, 0, 0 );
  if ( _cmd == SVN::Diff )
    mainBtn->setText( i18n( "&Show Diff" ) );
  else
    mainBtn->setText( i18n( "&Close" ) );
  connect( mainBtn, TQT_SIGNAL( clicked( ) ), this, TQT_SLOT( accept( ) ) );

  // Reenable the button and the log edit now that the process is finished.
  mainBtn->setEnabled( true );
  if ( _cmd == SVN::Commit )
    logedit->setEnabled( true );
}

TQString SVNDialog::statusOutput( )
{
  return _statusOutput;
}

void SVNDialog::readSettings( )
{
  TDESharedConfig * config = m_config;
  config->setGroup( "SVNSupport" );

  if ( _cmd == SVN::Commit ) {
    autoAddBox->setChecked( config->readBoolEntry( "AutoAddFiles", true ) );

    // Fill the combobox with old messages.
    m_logMessages.clear();
    m_squeezedLogMessages.clear();
    for ( int cnt = 0; cnt < 10; cnt++ )
      if ( config->hasKey( TQString( "CommitLogMessage%1" ).arg( cnt ) ) )
      {
        const TQString logMessage = config->readEntry( TQString( "CommitLogMessage%1" ).arg( cnt ) );
        if ( !logMessage.isEmpty() )
        {
          // If the message is too long, cut it to 80 characters (or the combo box becomes too wide)
          // ### FIXME: if the string matches the squeezed 80 chars, it might overwrite another entry
          const TQString shortLog = KStringHandler::csqueeze( logMessage );
          m_logMessages.append( logMessage );
          m_squeezedLogMessages.append( shortLog );
          oldMessages->insertItem( shortLog );
        }
      }

  }
}

void SVNDialog::saveSettings( )
{
  TDESharedConfig * config = m_config;
  config->setGroup( "SVNSupport" );
  if ( _cmd == SVN::Commit ) {
    config->writeEntry( "AutoAddFiles", autoAddBox->isChecked( ) );

    // Write the log messages to the config file.
    int cnt = 0;
    TQStringList::const_iterator it;
    for ( it = m_logMessages.constBegin( ); it != m_logMessages.constEnd( ) && cnt < 10 ; ++it, ++cnt )
      config->writeEntry( TQString( "CommitLogMessage%1" ).arg( cnt ), *it );
  }
  m_config->sync();
}

#include "svndialog.moc"

// kate: space-indent on; indent-width 2; replace-tabs on;
